/* PROGRAM  : dupfind.cmd
   AUTHOR   : N. Morrow - morrown@netscape.net
   LANGUAGE : REXX
   OS       : eCS v1.2
   SYNOPSIS : duplicate filename find utility

   USAGE    : dupfind <volume letter>

   EXAMPLE  : > dupfind x
                or
            : > dupfind x:

   USAGE    : dupfind /h or dupfind -h
            : displays help

   CREATED  : 2005 Apr 22
   UPDATED  : 2005 Jun 01
   STATUS   : Public Domain
   NOTES    : 

   dupfind checks for duplicate filenames on the selected volume.

   The results are logged to dupfind.log.  After processing is
   finished the location of dupfind.log will be displayed.  If 
   the environment variable LOGFILES is set, as it is by default
   in current versions of eCS, then dupfind will respect this
   setting and place dupfind.log in the location specified by
   LOGFILES.

   dupfind is a processor intensive utility.  On older systems
   with large volumes the time to process may be lengthy.

   My motivation for writing this little utility is because I am
   disappointed at the number of unneeded duplicate files that
   are contained in an eCS installation volume.  Give this utility
   a try on your eCS installation volume and see what I mean.  If you
   don't like that a tremendous amount of your drive space is being
   wasted with unnecessary duplicate files then contact your eCS
   supplier and make it known that this is unacceptable.

   Examples:

   06/09/04   6:16p     1675430  \ecs\install\PRNDRV\PMDD_0\LASERJET.DRV
   06/09/04   6:16p     1675430  \OS2\DLL\LASERJET\LASERJET.DRV

   07/16/03  10:40a      153088  \ecs\system\thememgr\PMMRGRES.DLL
   07/16/03  10:40a      153088  \OS2\DLL\PMMRGRES.DLL

   07/13/01   3:58p      111759  \ecs\install\PRNDRV\PMDD_0\ibmnull.drv
   07/13/01   3:58p      111759  \OS2\DLL\IBMNULL\IBMNULL.DRV

   01/28/04   4:14p      847951  \ecs\dll\WPIRTL.DLL
   07/14/04  11:25p      847951  \ecs\install\WarpIN\wpirtl.dll

   History:

   v1.02 - dupfind now prompts for the volume letter if none is given.
         - changed to use LOGFILES if it is set.
         - added a beep to signal processing complete.

   v1.05 - show accessible local volumes when in interactive mode
         - general code clean up

*/

/* set program name and version */
progVer = "dupfind v1.05";

/* add required functions from REXXUTIL */
rc = RXFUNCQUERY(SysCurState);
if rc \= 0 then
    do
        rc = RxFuncAdd("SysCurState","RexxUtil","SYSCURSTATE");
        if rc \= 0 then
            do
                say "Error: Unable to initialize the REXX SysCurState function";
                exit 1;
            end
    end

rc = RXFUNCQUERY(SysDriveInfo);
if rc \= 0 then
    do
        rc = RxFuncAdd("SysDriveInfo","RexxUtil","SYSDRIVEINFO");
        if rc \= 0 then
            do
                say "Error: Unable to initialize the REXX SysDriveInfo function";
                exit 1;
            end
    end

rc = RXFUNCQUERY(SysDriveMap);
if rc \= 0 then
    do
        rc = RxFuncAdd("SysDriveMap","RexxUtil","SYSDRIVEMAP");
        if rc \= 0 then
            do
                say "Error: Unable to initialize the REXX SysDriveMap function";
                exit 1;
            end
    end

rc = RXFUNCQUERY(SysFileDelete);
if rc \= 0 then
    do
        rc = RxFuncAdd("SysFileDelete","RexxUtil","SYSFILEDELETE");
        if rc \= 0 then
            do
                say "Error: Unable to initialize the REXX SysFileDelete function";
                exit 1;
            end
    end

rc = RXFUNCQUERY(SysFileTree);
if rc \= 0 then
    do
        rc = RxFuncAdd("SysFileTree","RexxUtil","SYSFILETREE");
        if rc \= 0 then
            do
                say "Error: Unable to initialize the REXX SysFileTree function";
                exit 1;
            end
    end

rc = RXFUNCQUERY(SysGetKey);
if rc \= 0 then
    do
        rc = RxFuncAdd("SysGetKey","RexxUtil","SYSGETKEY");
        if rc \= 0 then
            do
                say "Error: Unable to initialize the REXX SysGetKey function";
                exit 1;
            end
    end

/* get the volume letter entered at the command line and make it upper case */
parse upper arg volumeLetter

/* display help */
if (POS("/H",volumeLetter) <> 0) |,
   (POS("-H",volumeLetter) <> 0) |,
   (POS('?', volumeLetter) <> 0) then
    do /* output help screen */
        say
        say progVer;
        say
        say "PURPOSE: Find duplicate filenames.";
        say
        say "USAGE  : dupfind <volume letter>";
        say
        say "EXAMPLE: > dupfind x";
        say "           or";
        say "       : > dupfind x:";
        return 1;
    end

/* determine accessible local volumes */
localVolumes = SysDriveMap("C:", "LOCAL");

/* ask for volume letter if it wasn't supplied */
if (LENGTH(volumeLetter) < 1) then
    do
        say
        say progVer;
        say
        say "Accessible local volumes: " localVolumes; 
        say
        call CharOut, "Which volume would you like to check? ";
        pull volumeLetter;
    end

/* add ':' if the user didn't */
if RIGHT(volumeLetter,1) \= ':' then volumeLetter = volumeLetter||':'


/* display error if non-existant volume is entered */
rc = SysDriveInfo(volumeLetter);
if rc = '' then
    do
        say
        say "ERROR: The volume you selected appears not to exist.";
        return 1;
    end  

/* use value in LOGFILES environment variable if it is set */
outputDir = VALUE("LOGFILES",,"OS2ENVIRONMENT");

/* check if directory in LOGFILES exist */
rc = DIRECTORY(outputDir);
if rc \= '' then
    if RIGHT(outputDir,1) = '\' then
        outputFile = outputDir||"dupfind.log";
    else
        outputFile = outputDir||"\dupfind.log";
else
    do
        say
        say "ERROR: The directory in environment variable LOGFILES appears not to exist.";
        return 1;
    end  

/* if LOGFILES is not set then use current directory */
if outputDir = '' then
    do   
        outputDir = directory();
        if RIGHT(outputDir,1) = '\' then
            outputFile = outputDir||"dupfind.log";
        else
            outputFile = outputDir||"\dupfind.log";
    end

/* delete existing dupfind.log */
if STREAM(outputFile, 'C', "QUERY EXISTS") \= '' then
    call SysFileDelete outputFile;

/* dupfind can take a long time on older systems */
say
call SysCurState "OFF"
say "Working...";


/* reads in all filenames from volume specified, ignors hidden files */
rc = SysFileTree(volumeLetter||"\*.*", "fullyQualifiedFilenames", "FS", "**-**");

/* display error code */
if rc \= 0 then
    do
        say
        say "ERROR: SysFileTree reported error code" rc;
        return 1;
    end  


/* we need to work with filenames, not fully qualified filenames */
do i = 1 to fullyQualifiedFilenames.0 
    filenames.i = TRANSLATE(FILESPEC("name", fullyQualifiedFilenames.i));
end

/* find and output duplicate filenames */
i = 0;
k = 0;
dupFilenames. = '';
do x = 1 to fullyQualifiedFilenames.0
    alreadyDone = 0;
    j = 0;
    testFilenames.x = filenames.x;
    if k > 0 then
        do m = 1 to k
            if testFilenames.x = dupFilenames.m then alreadyDone = 1;
        end
    if alreadyDone = 1 then nop
    else
        do z = (x + 1) to fullyQualifiedFilenames.0
            if filenames.z = testFilenames.x then
                do
                    k = k + 1;
                    dupFilenames.k = testFilenames.x;
                    i = i + 1;
                    if j = 0 then call LINEOUT outputFile, fullyQualifiedFilenames.x;
                    j = j + 1;
                    call LINEOUT outputFile, fullyQualifiedFilenames.z;
                end
        end
    if j > 0 then call LINEOUT outputFile, '' 
end

/* tell user we are finished and display output file location */
say
say "Finished.";
call BEEP 800,25;
say
say "Output file: " outputFile;
say
say "Press any key to continue.";
inkey = SysGetKey('NOECHO')

/* return 0 to os signifying sucessful finish */
return 0;

